#include <linux/fb.h>

const struct fb_videomode cea_modes[65] = {
	[1] = {
		.name = "CEA VIC 1: 640x480@60",
		.refresh = 60,
		.xres = 640,
		.yres = 480,
		.pixclock = 39721, /* 25.175MHz */
		.left_margin = 48,
		.right_margin = 16,
		.upper_margin = 33,
		.lower_margin = 12,
		.hsync_len = 96,
		.vsync_len = 2,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[2] = {
		.name = "CEA VIC 2: 720x480@60",
		.refresh = 60,
		.xres = 720,
		.yres = 480,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 60,
		.right_margin = 16,
		.upper_margin = 30,
		.lower_margin = 15,
		.hsync_len = 62,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[3] = {
		.name = "CEA VIC 3: 720x480@60",
		.refresh = 60,
		.xres = 720,
		.yres = 480,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 60,
		.right_margin = 16,
		.upper_margin = 30,
		.lower_margin = 15,
		.hsync_len = 62,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[4] = {
		.name = "CEA VIC 4: 1280x720@60",
		.refresh = 60,
		.xres = 1280,
		.yres = 720,
		.pixclock = 13468, /* 74.250MHz */
		.left_margin = 220,
		.right_margin = 110,
		.upper_margin = 20,
		.lower_margin = 10,
		.hsync_len = 40,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[5] = {
		.name = "CEA VIC 5: 1920x1080i@30",
		.refresh = 30,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 13468, /* 74.250MHz */
		.left_margin = 148,
		.right_margin = 88,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 44,
		.vsync_len = 10,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED,
		.flag = FB_MODE_IS_CEA,
	},
	[6] = {
		.name = "CEA VIC 6: 1440x480i@30",
		.refresh = 30,
		.xres = 1440,
		.yres = 480,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 114,
		.right_margin = 38,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 124,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[7] = {
		.name = "CEA VIC 7: 1440x480i@30",
		.refresh = 30,
		.xres = 1440,
		.yres = 480,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 114,
		.right_margin = 38,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 124,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[8] = {
		.name = "CEA VIC 8: 1440x240@60",
		.refresh = 60,
		.xres = 1440,
		.yres = 240,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 114,
		.right_margin = 38,
		.upper_margin = 15,
		.lower_margin = 7,
		.hsync_len = 124,
		.vsync_len = 3,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[9] = {
		.name = "CEA VIC 9: 1440x240@60",
		.refresh = 60,
		.xres = 1440,
		.yres = 240,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 114,
		.right_margin = 38,
		.upper_margin = 15,
		.lower_margin = 7,
		.hsync_len = 124,
		.vsync_len = 3,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[10] = {
		.name = "CEA VIC 10: 2880x480i@30",
		.refresh = 30,
		.xres = 2880,
		.yres = 480,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 228,
		.right_margin = 76,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 248,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[11] = {
		.name = "CEA VIC 11: 2880x480i@30",
		.refresh = 30,
		.xres = 2880,
		.yres = 480,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 228,
		.right_margin = 76,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 248,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[12] = {
		.name = "CEA VIC 12: 2880x240@60",
		.refresh = 60,
		.xres = 2880,
		.yres = 240,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 228,
		.right_margin = 76,
		.upper_margin = 15,
		.lower_margin = 7,
		.hsync_len = 248,
		.vsync_len = 3,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[13] = {
		.name = "CEA VIC 13: 2880x240@60",
		.refresh = 60,
		.xres = 2880,
		.yres = 240,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 228,
		.right_margin = 76,
		.upper_margin = 15,
		.lower_margin = 7,
		.hsync_len = 248,
		.vsync_len = 3,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[14] = {
		.name = "CEA VIC 14: 1440x480@60",
		.refresh = 60,
		.xres = 1440,
		.yres = 480,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 120,
		.right_margin = 32,
		.upper_margin = 30,
		.lower_margin = 15,
		.hsync_len = 124,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[15] = {
		.name = "CEA VIC 15: 1440x480@60",
		.refresh = 60,
		.xres = 1440,
		.yres = 480,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 120,
		.right_margin = 32,
		.upper_margin = 30,
		.lower_margin = 15,
		.hsync_len = 124,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[16] = {
		.name = "CEA VIC 16: 1920x1080@60",
		.refresh = 60,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 6734, /* 148.500MHz */
		.left_margin = 148,
		.right_margin = 88,
		.upper_margin = 36,
		.lower_margin = 9,
		.hsync_len = 44,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[17] = {
		.name = "CEA VIC 17: 720x576@50",
		.refresh = 50,
		.xres = 720,
		.yres = 576,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 68,
		.right_margin = 12,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 64,
		.vsync_len = 5,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[18] = {
		.name = "CEA VIC 18: 720x576@50",
		.refresh = 50,
		.xres = 720,
		.yres = 576,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 68,
		.right_margin = 12,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 64,
		.vsync_len = 5,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[19] = {
		.name = "CEA VIC 19: 1280x720@50",
		.refresh = 50,
		.xres = 1280,
		.yres = 720,
		.pixclock = 13468, /* 74.250MHz */
		.left_margin = 220,
		.right_margin = 440,
		.upper_margin = 20,
		.lower_margin = 10,
		.hsync_len = 40,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[20] = {
		.name = "CEA VIC 20: 1920x1080i@25",
		.refresh = 25,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 13468, /* 74.250MHz */
		.left_margin = 148,
		.right_margin = 528,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 44,
		.vsync_len = 10,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED,
		.flag = FB_MODE_IS_CEA,
	},
	[21] = {
		.name = "CEA VIC 21: 1440x576i@25",
		.refresh = 25,
		.xres = 1440,
		.yres = 576,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 138,
		.right_margin = 24,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 126,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[22] = {
		.name = "CEA VIC 22: 1440x576i@25",
		.refresh = 25,
		.xres = 1440,
		.yres = 576,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 138,
		.right_margin = 24,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 126,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[23] = {
		.name = "CEA VIC 23: 1440x288@50",
		.refresh = 50,
		.xres = 1440,
		.yres = 288,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 138,
		.right_margin = 24,
		.upper_margin = 19,
		.lower_margin = 5,
		.hsync_len = 126,
		.vsync_len = 3,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[24] = {
		.name = "CEA VIC 24: 1440x288@50",
		.refresh = 50,
		.xres = 1440,
		.yres = 288,
		.pixclock = 37037, /* 27.0MHz */
		.left_margin = 138,
		.right_margin = 24,
		.upper_margin = 19,
		.lower_margin = 5,
		.hsync_len = 126,
		.vsync_len = 3,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[25] = {
		.name = "CEA VIC 25: 2880x576i@25",
		.refresh = 25,
		.xres = 2880,
		.yres = 576,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 276,
		.right_margin = 48,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 252,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[26] = {
		.name = "CEA VIC 26: 2880x576i@25",
		.refresh = 25,
		.xres = 2880,
		.yres = 576,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 276,
		.right_margin = 48,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 252,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[27] = {
		.name = "CEA VIC 27: 2880x288@50",
		.refresh = 50,
		.xres = 2880,
		.yres = 288,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 276,
		.right_margin = 48,
		.upper_margin = 19,
		.lower_margin = 5,
		.hsync_len = 252,
		.vsync_len = 3,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[28] = {
		.name = "CEA VIC 28: 2880x288@50",
		.refresh = 50,
		.xres = 2880,
		.yres = 288,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 276,
		.right_margin = 48,
		.upper_margin = 19,
		.lower_margin = 5,
		.hsync_len = 252,
		.vsync_len = 3,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[29] = {
		.name = "CEA VIC 29: 1440x576@50",
		.refresh = 50,
		.xres = 1440,
		.yres = 576,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 136,
		.right_margin = 24,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 128,
		.vsync_len = 5,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[30] = {
		.name = "CEA VIC 30: 1440x576@50",
		.refresh = 50,
		.xres = 1440,
		.yres = 576,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 136,
		.right_margin = 24,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 128,
		.vsync_len = 5,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[31] = {
		.name = "CEA VIC 31: 1920x1080@50",
		.refresh = 50,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 6734, /* 148.500MHz */
		.left_margin = 148,
		.right_margin = 528,
		.upper_margin = 36,
		.lower_margin = 9,
		.hsync_len = 44,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[32] = {
		.name = "CEA VIC 32: 1920x1080@24",
		.refresh = 24,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 13468, /* 74.250MHz */
		.left_margin = 148,
		.right_margin = 638,
		.upper_margin = 36,
		.lower_margin = 9,
		.hsync_len = 44,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[33] = {
		.name = "CEA VIC 33: 1920x1080@25",
		.refresh = 25,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 13468, /* 74.250MHz */
		.left_margin = 148,
		.right_margin = 528,
		.upper_margin = 36,
		.lower_margin = 9,
		.hsync_len = 44,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[34] = {
		.name = "CEA VIC 34: 1920x1080@30",
		.refresh = 30,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 13468, /* 74.250MHz */
		.left_margin = 148,
		.right_margin = 88,
		.upper_margin = 36,
		.lower_margin = 9,
		.hsync_len = 44,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[35] = {
		.name = "CEA VIC 35: 2880x480@60",
		.refresh = 60,
		.xres = 2880,
		.yres = 480,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 240,
		.right_margin = 64,
		.upper_margin = 30,
		.lower_margin = 15,
		.hsync_len = 248,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[36] = {
		.name = "CEA VIC 36: 2880x480@60",
		.refresh = 60,
		.xres = 2880,
		.yres = 480,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 240,
		.right_margin = 64,
		.upper_margin = 30,
		.lower_margin = 15,
		.hsync_len = 248,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[37] = {
		.name = "CEA VIC 37: 2880x576@50",
		.refresh = 50,
		.xres = 2880,
		.yres = 576,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 272,
		.right_margin = 48,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 256,
		.vsync_len = 5,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[38] = {
		.name = "CEA VIC 38: 2880x576@50",
		.refresh = 50,
		.xres = 2880,
		.yres = 576,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 272,
		.right_margin = 48,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 256,
		.vsync_len = 5,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[39] = {
		.name = "CEA VIC 39: 1920x1080i@25",
		.refresh = 25,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 13888, /* 72.0MHz */
		.left_margin = 184,
		.right_margin = 32,
		.upper_margin = 114,
		.lower_margin = 56,
		.hsync_len = 168,
		.vsync_len = 10,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED,
		.flag = FB_MODE_IS_CEA,
	},
	[40] = {
		.name = "CEA VIC 40: 1920x1080i@50",
		.refresh = 50,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 6734, /* 148.500MHz */
		.left_margin = 148,
		.right_margin = 528,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 44,
		.vsync_len = 10,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED,
		.flag = FB_MODE_IS_CEA,
	},
	[41] = {
		.name = "CEA VIC 41: 1280x720@100",
		.refresh = 100,
		.xres = 1280,
		.yres = 720,
		.pixclock = 6734, /* 148.500MHz */
		.left_margin = 220,
		.right_margin = 440,
		.upper_margin = 20,
		.lower_margin = 10,
		.hsync_len = 40,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[42] = {
		.name = "CEA VIC 42: 720x576@100",
		.refresh = 100,
		.xres = 720,
		.yres = 576,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 68,
		.right_margin = 12,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 64,
		.vsync_len = 5,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[43] = {
		.name = "CEA VIC 43: 720x576@100",
		.refresh = 100,
		.xres = 720,
		.yres = 576,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 68,
		.right_margin = 12,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 64,
		.vsync_len = 5,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[44] = {
		.name = "CEA VIC 44: 1440x576@50",
		.refresh = 50,
		.xres = 1440,
		.yres = 576,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 138,
		.right_margin = 24,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 126,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[45] = {
		.name = "CEA VIC 45: 1440x576@50",
		.refresh = 50,
		.xres = 1440,
		.yres = 576,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 138,
		.right_margin = 24,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 126,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
		.vmode = FB_VMODE_DOUBLE,
	},
	[46] = {
		.name = "CEA VIC 46: 1920x1080i@60",
		.refresh = 60,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 6734, /* 148.500MHz */
		.left_margin = 148,
		.right_margin = 88,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 44,
		.vsync_len = 10,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED,
		.flag = FB_MODE_IS_CEA,
	},
	[47] = {
		.name = "CEA VIC 47: 1280x720@120",
		.refresh = 120,
		.xres = 1280,
		.yres = 720,
		.pixclock = 6734, /* 148.500MHz */
		.left_margin = 220,
		.right_margin = 110,
		.upper_margin = 20,
		.lower_margin = 10,
		.hsync_len = 40,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[48] = {
		.name = "CEA VIC 48: 720x480@119",
		.refresh = 119,
		.xres = 720,
		.yres = 480,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 60,
		.right_margin = 16,
		.upper_margin = 30,
		.lower_margin = 15,
		.hsync_len = 62,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[49] = {
		.name = "CEA VIC 49: 720x480@119",
		.refresh = 119,
		.xres = 720,
		.yres = 480,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 60,
		.right_margin = 16,
		.upper_margin = 30,
		.lower_margin = 15,
		.hsync_len = 62,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[50] = {
		.name = "CEA VIC 50: 1440x480i@60",
		.refresh = 60,
		.xres = 1440,
		.yres = 480,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 114,
		.right_margin = 38,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 124,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[51] = {
		.name = "CEA VIC 51: 1440x480i@60",
		.refresh = 60,
		.xres = 1440,
		.yres = 480,
		.pixclock = 18518, /* 54.0MHz */
		.left_margin = 114,
		.right_margin = 38,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 124,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[52] = {
		.name = "CEA VIC 52: 720x576@199",
		.refresh = 199,
		.xres = 720,
		.yres = 576,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 68,
		.right_margin = 12,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 64,
		.vsync_len = 5,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[53] = {
		.name = "CEA VIC 53: 720x576@199",
		.refresh = 199,
		.xres = 720,
		.yres = 576,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 68,
		.right_margin = 12,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 64,
		.vsync_len = 5,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[54] = {
		.name = "CEA VIC 54: 1440x576i@100",
		.refresh = 100,
		.xres = 1440,
		.yres = 576,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 138,
		.right_margin = 24,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 126,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[55] = {
		.name = "CEA VIC 55: 1440x576i@100",
		.refresh = 100,
		.xres = 1440,
		.yres = 576,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 138,
		.right_margin = 24,
		.upper_margin = 39,
		.lower_margin = 10,
		.hsync_len = 126,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED  | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[56] = {
		.name = "CEA VIC 56: 720x480@238",
		.refresh = 238,
		.xres = 720,
		.yres = 480,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 60,
		.right_margin = 16,
		.upper_margin = 30,
		.lower_margin = 15,
		.hsync_len = 62,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[57] = {
		.name = "CEA VIC 57: 720x480@238",
		.refresh = 238,
		.xres = 720,
		.yres = 480,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 60,
		.right_margin = 16,
		.upper_margin = 30,
		.lower_margin = 15,
		.hsync_len = 62,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[58] = {
		.name = "CEA VIC 58: 1440x480i@119",
		.refresh = 119,
		.xres = 1440,
		.yres = 480,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 114,
		.right_margin = 38,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 124,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[59] = {
		.name = "CEA VIC 59: 1440x480i@119",
		.refresh = 119,
		.xres = 1440,
		.yres = 480,
		.pixclock = 9259, /* 108.0MHz */
		.left_margin = 114,
		.right_margin = 38,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 124,
		.vsync_len = 6,
		.sync = FB_SYNC_BROADCAST,
		.vmode = FB_VMODE_INTERLACED | FB_VMODE_DOUBLE,
		.flag = FB_MODE_IS_CEA,
	},
	[60] = {
		.name = "CEA VIC 60: 1280x720@24",
		.refresh = 24,
		.xres = 1280,
		.yres = 720,
		.pixclock = 16835, /* 59.400MHz */
		.left_margin = 220,
		.right_margin = 1760,
		.upper_margin = 20,
		.lower_margin = 10,
		.hsync_len = 40,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[61] = {
		.name = "CEA VIC 61: 1280x720@25",
		.refresh = 25,
		.xres = 1280,
		.yres = 720,
		.pixclock = 13468, /* 74.250MHz */
		.left_margin = 220,
		.right_margin = 2420,
		.upper_margin = 20,
		.lower_margin = 10,
		.hsync_len = 40,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[62] = {
		.name = "CEA VIC 62: 1280x720@30",
		.refresh = 30,
		.xres = 1280,
		.yres = 720,
		.pixclock = 13468, /* 74.250MHz */
		.left_margin = 220,
		.right_margin = 1760,
		.upper_margin = 20,
		.lower_margin = 10,
		.hsync_len = 40,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[63] = {
		.name = "CEA VIC 63: 1920x1080@120",
		.refresh = 120,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 3367, /* 297.0MHz */
		.left_margin = 148,
		.right_margin = 88,
		.upper_margin = 36,
		.lower_margin = 9,
		.hsync_len = 44,
		.vsync_len = 5,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
	[64] = {
		.name = "CEA VIC 64: 1920x1080@100",
		.refresh = 100,
		.xres = 1920,
		.yres = 1080,
		.pixclock = 3367, /* 297.0MHz */
		.left_margin = 148,
		.right_margin = 528,
		.upper_margin = 31,
		.lower_margin = 14,
		.hsync_len = 44,
		.vsync_len = 10,
		.sync = FB_SYNC_HOR_HIGH_ACT | FB_SYNC_HOR_HIGH_ACT | FB_SYNC_BROADCAST,
		.flag = FB_MODE_IS_CEA,
	},
};
